/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: InstallationPackage.java 114 2006-03-23 11:33:07Z alouis $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.packaging;

import java.io.IOException;
import java.net.URI;
import java.util.List;
import java.util.zip.ZipFile;

import org.objectweb.petals.PetalsException;
import org.objectweb.petals.tools.jbicommon.descriptor.ComponentDescription;
import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptor;
import org.objectweb.petals.tools.jbicommon.descriptor.SharedLibrary;
import org.objectweb.petals.util.ZipUtil;
import org.objectweb.util.monolog.api.Logger;

/**
 * JBI Installation Package. This object should be constructed via
 * JBIPackageBuilder.
 * 
 * @version $Rev: 114 $ $Date: 2006-03-23 12:33:07 +0100 (jeu., 23 mars 2006) $
 * @since Petals 1.0
 * @author <a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 */
class InstallationPackage extends AbstractJBIPackage {

    /**
     * Is a component installation package.
     */
    private Boolean componentInstallation;

    /**
     * @param packageZipURI
     * @param descriptor
     * @throws PetalsException
     */
    public InstallationPackage(URI packageZipURI, JBIDescriptor descriptor,
            Logger logger) {
        super(packageZipURI, descriptor, logger);
    }

    private static void checkClassPathElements(ZipFile archive,
            List<String> classPath, String production) throws PetalsException {
        String msg;

        for (String pathElement : classPath) {

            // empty class-path elements is not allowed
            if (pathElement == null || "".equals(pathElement.trim())) {
                msg = "An empty " + production + " class-path element found "
                        + "in the JBI Descriptor file (jbi.xml).";
                throw new PetalsException(msg);
            }

            // check if the specified path element exists within archive
            if (!ZipUtil.hasEntry(archive, pathElement)) {
                // specified class path element not found within archive
                msg = "A " + production + " class-path element declared in "
                        + "JBI Descriptor not found within installation "
                        + "package Zip archive file.";
                throw new PetalsException(msg);
            }

        }

    }

    // ----------------------------------------------------------------------
    // implementation specific methods
    // ----------------------------------------------------------------------

    /**
     * Only check consistency against class-path elements and the corresponding
     * entry within the installation package Zip archive file. Checks relative
     * to shared-library-list in component production should occur further
     * during the installation process.
     * 
     * @see org.objectweb.petals.jbi.management.packaging.JBIPackage#checkContents()
     */
    public void checkContents() throws PetalsException {
        log.start();

        JBIDescriptor descriptor = getDescriptor();

        ZipFile zipArchive = ZipUtil.openZipFile(getArchiveURI());

        try {

            // component installation checks
            if (isComponentInstallation()) {
                ComponentDescription component = descriptor.getComponent();
                checkClassPathElements(zipArchive, component
                        .getComponentClassPath(), "component");
                checkClassPathElements(zipArchive, component
                        .getBootstrapClassPath(), "bootstrap");
            } else {
                // shared library checks
                SharedLibrary sl = descriptor.getSharedLibrary();

                checkClassPathElements(zipArchive, sl
                        .getSharedLibraryClassPath(), "shared-library");
            }
        } finally {
            try {
                zipArchive.close();
            } catch (IOException e) {
                throw new PetalsException(
                        "Error while closing the Zip archive file.", e);
            }
        }
        log.end();
    }

    public boolean isComponentInstallation() {
        log.start();

        if (componentInstallation == null) {
            JBIDescriptor descriptor = getDescriptor();

            componentInstallation = Boolean
                    .valueOf(descriptor.getComponent() != null);
        }
        log.end();

        return componentInstallation.booleanValue();
    }

}
