/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JBIPackageBuilder.java 114 2006-03-23 11:33:07Z alouis $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.packaging;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.zip.ZipFile;

import org.objectweb.petals.PetalsException;
import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptor;
import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptorBuilder;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.petals.util.ZipUtil;
import org.objectweb.util.monolog.api.Logger;
import org.xml.sax.SAXException;

/**
 * JBIPackage Builder.
 * 
 * @version $Rev: 114 $ $Date: 2006-03-23 12:33:07 +0100 (jeu., 23 mars 2006) $
 * @since Petals 1.0
 * @author <a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 */
public class JBIPackageBuilder {

    /**
     * Logger wrapper
     */
    private LoggingUtil log;

    /**
     * Monolog logger
     */
    private Logger logger;

    /**
     * Package URI location.
     */
    private URI packageLocation;

    /**
     * Constructor with a given package Zip archive URL to build a JBIPackage
     * from that source.
     * 
     * @throws PetalsException
     *             wrong package ZIP archive URL were specified.
     */
    public JBIPackageBuilder(String packageZipURL, Logger logger)
        throws PetalsException {
        log = new LoggingUtil(logger);

        packageLocation = processAndGetPackageURI(packageZipURL);
    }

    // ----------------------------------------------------------------------
    // JBIPackageBuilder methods
    // ----------------------------------------------------------------------

    /**
     * Build the JBIPackage specified by the given Zip archive location.
     */
    public JBIPackage build() throws PetalsException {
        log.start();

        JBIPackage pkg = null;

        JBIDescriptor descriptor = loadDescriptor(packageLocation);

        // build a Deployment Package
        if (AbstractJBIPackage.isDeploymentPackage(descriptor)) {
            pkg = new DeploymentPackage(packageLocation, descriptor, logger);

            // builds an Installation Package
        } else {
            pkg = new InstallationPackage(packageLocation, descriptor, logger);
        }

        // extra validation empowered by JBIPackage
        try {
            pkg.checkContents();
        } catch (PetalsException pe) {
            log.error(
                "An error found checking the package Zip archive contents. "
                    + pe.getMessage(), pe);
            throw pe;
        }
        log.end();

        return pkg;
    }

    /**
     * 
     * @param archiveURI
     * @return
     * @throws PetalsException
     */
    private JBIDescriptor loadDescriptor(URI archiveURI) throws PetalsException {
        log.start();

        String msg;
        JBIDescriptor result = null;

        // extract descriptor file to parse and validate

        File tmpJbiXml = null;

        try {
            // open the package Zip archive file and extract jbi.xml temp
            try {
                ZipFile zipArchive = ZipUtil.openZipFile(archiveURI);

                tmpJbiXml = ZipUtil.getEntryAsTemp(zipArchive,
                    JBIDescriptorBuilder.JBI_DESCRIPTOR_RESOURCE);

                zipArchive.close();
            } catch (IOException e) {
                // do nothing
            }

            if (tmpJbiXml == null) {
                msg = "Invalid JBI package Zip archive. jbi.xml descriptor "
                    + "entry was not found: " + archiveURI.toString();
                log.error(msg);

                throw new PetalsException(msg);
            }

            // use builder to parse and validate jbi.xml against the schema

            JBIDescriptorBuilder descBuilder = new JBIDescriptorBuilder(
                tmpJbiXml.toURI(), (java.util.logging.Logger) logger);

            result = descBuilder.build();

        } catch (SAXException saxe) {
            msg = "XML error parsing JBI descriptor. Package archive: "
                + archiveURI.toString();

            log.error(msg, saxe);

            throw new PetalsException(msg, saxe);
        } finally {
            // delete temporary file
            if (tmpJbiXml != null) {
                tmpJbiXml.delete();
            }
        }
        log.end();

        return result;
    }

    /**
     * Process the given package Zip archive URL string and transform into a
     * ready-to-use URI object. Additionally, checks if the point specified
     * exist.
     * 
     * @return <code>URI</code> the proper package URI.
     * @throws PetalsException
     *             wrong package ZIP archive URL were specified.
     */
    private URI processAndGetPackageURI(String packageZipURL)
        throws PetalsException {
        log.start(packageZipURL);

        String msg;
        URI uri = null;

        try {
            URL url = new URL(packageZipURL);

            uri = url.toURI();
        } catch (MalformedURLException mue) {
            msg = "Wrong package archive location specified. URL: "
                + packageZipURL;

            log.error(msg, mue);

            throw new PetalsException(msg, mue);
        } catch (URISyntaxException use) {
            msg = "Syntax error converting to URI from given location. URL: "
                + packageZipURL;
            log.error(msg, use);

            throw new PetalsException(msg, use);
        }

        // check if exists
        File archive = new File(uri);

        if (!archive.exists()) {
            msg = "JBI package archive not found. URL: " + packageZipURL;
            log.error(msg);

            throw new PetalsException(msg);
        }
        log.end();

        return uri;
    }

}
