/**
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: JoramTransporter.java,v 0.3 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.jbi.transport;

import javax.jbi.JBIException;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.ObjectMessage;
import javax.jms.QueueConnection;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.naming.InitialContext;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.joram.client.jms.Queue;
import org.objectweb.petals.jbi.messaging.MessageExchangeImpl;
import org.objectweb.petals.jbi.routing.Router;
import org.objectweb.petals.jbi.routing.RoutingException;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.petals.util.PropertyUtil;
import org.objectweb.petals.util.SystemUtil;
import org.objectweb.util.monolog.api.Logger;

import static org.objectweb.petals.util.InterfaceNamesHelper.JBI_ROUTER_NAME;
import static org.objectweb.petals.util.InterfaceNamesHelper.LOGGER_NAME;

/**
 * Implementation of the <code>Transporter</code> with JORAM. <br>
 * This implementation uses the fractal JoramAgent. <br>
 * Each <code>JoramTransporter</code> registrers a <code>Queue</code> in the
 * Joram distributed JNDI with the following name :
 * <em>JoramQueue-[containerId]</em> <br>
 * The name of a container is the <em>ID</em> specified in the
 * <em>conf/server.properties</em> file. <br>
 * This <code>JoramTransporter</code> uses an <code>ObjectSerializer</code>
 * to create <em>jms.ObjectMessage</em> to send via JMS. So, the
 * <code>MessageExchange</code> to send is serialized.
 * 
 * @version $Rev: 477 $ $Date: 2006-05-29 17:18:07 +0200 (lun., 29 mai 2006) $
 * @since Petals 1.0
 * @author alouis
 */
public class JoramTransporter implements Transporter, BindingController,
    LifeCycleController {

    private static final String[] CLIENT_INTERFACES = new String[] {JBI_ROUTER_NAME};

    /**
     * Initial context
     */
    protected InitialContext context;

    /**
     * Component LifeCycle state.
     */
    protected String fcState;

    protected String host;

    protected int id;

    protected JoramQueue joramQueue;

    /**
     * Logger
     */
    protected LoggingUtil log;

    protected String pwd;

    protected Router router;

    protected Serializer serializer = new ObjectSerializer();

    protected int tcp;

    protected String user;

    /**
     * @see org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.String,
     *      java.lang.Object)
     */
    public void bindFc(String clientItfName, Object serverItf)
        throws NoSuchInterfaceException, IllegalBindingException,
        IllegalLifeCycleException {
        if (LOGGER_NAME.equals(clientItfName)) {
            log = new LoggingUtil((Logger) serverItf);
        } else if (JBI_ROUTER_NAME.equals(clientItfName)) {
            router = (Router) serverItf;
        } else {
            throw new NoSuchInterfaceException(clientItfName);
        }
    }

    /**
     * @see org.objectweb.fractal.api.control.LifeCycleController#getFcState()
     */
    public String getFcState() {
        return STARTED.equals(fcState) ? STARTED : STOPPED;
    }

    /**
     * @see org.objectweb.fractal.api.control.BindingController#listFc()
     */
    public String[] listFc() {
        return CLIENT_INTERFACES;
    }

    /**
     * @see org.objectweb.fractal.api.control.BindingController#lookupFc(java.lang.String)
     */
    public Object lookupFc(String clientItfName)
        throws NoSuchInterfaceException {
        if (JBI_ROUTER_NAME.equals(clientItfName)) {
            return router;
        } else if (LOGGER_NAME.equals(clientItfName)) {
            return log;
        } else {
            throw new NoSuchInterfaceException(clientItfName);
        }
    }

    /**
     * Send the message to the specified container. The send can be local or
     * distant TODO the transporter has to know its own containername to make
     * optimization and perform a local send.
     */
    public void send(MessageExchangeImpl messageExchange, String containerName)
        throws TransportException {
        sendDistant(messageExchange, containerName);
        /*
         * log.start(); // If consumer and provider are on the same container, //
         * by pass JSM String consumerContainerName =
         * messageExchange.getConsumerEndpoint().getContainerName(); // not OK
         * for provider->consumer
         * if(consumerContainerName.equals(containerName)) {
         * sendLocal(messageExchange); } else {
         * sendDistant(messageExchange,containerName); } log.end();
         */
    }

    /**
     * @see org.objectweb.fractal.api.control.LifeCycleController#startFc()
     */
    public void startFc() throws IllegalLifeCycleException {
        try {
            initJMS();
            startJMS();
            fcState = STARTED;
        } catch (Exception e) {
            log.error(e);
        }
    }

    /**
     * @see org.objectweb.fractal.api.control.LifeCycleController#stopFc()
     */
    public void stopFc() throws IllegalLifeCycleException {
        // FIXME problem because the JNDI directory is already stopped
        // try {
        // stopJMS();
        // } catch (JBIException e) {
        // e.printStackTrace();
        // }
        fcState = STOPPED;
    }

    // ----------------------------------------------------------------------
    // Fractal BindingController implementation
    // ----------------------------------------------------------------------

    /**
     * @see org.objectweb.fractal.api.control.BindingController#unbindFc(java.lang.String)
     */
    public void unbindFc(String clientItfName) throws NoSuchInterfaceException,
        IllegalBindingException, IllegalLifeCycleException {
        if (LOGGER_NAME.equals(clientItfName)) {
            log = null;
        } else if (JBI_ROUTER_NAME.equals(clientItfName)) {
            router = null;
        } else {
            throw new NoSuchInterfaceException(clientItfName);
        }
    }

    /**
     * Create JoramQueue
     * 
     * @return JoramQueue
     */
    protected JoramQueue createJoramQueue() {
        return new JoramQueue(id, tcp, user, pwd, host, context);
    }

    /**
     * init variables with joram.properties file and create queue and user for
     * this joram server
     * 
     * @throws Exception
     */
    protected void initJMS() throws Exception {
        log.start();
        // id = Integer.parseInt(PropertyUtil.getProperty(
        // PropertyUtil.SERVER_PROPS, "joram.id"));
        id = Integer.parseInt(SystemUtil.getContainerName());

        tcp = Integer.parseInt(SystemUtil.getJoramTCPPort());
        user = SystemUtil.getJoramUser();
        pwd = SystemUtil.getJoramPassword();
        host = SystemUtil.getHost();

        // create initialContext
        context = new javax.naming.InitialContext(PropertyUtil
            .retrieveJNDIProperties());

        joramQueue = createJoramQueue();
        joramQueue.init();

        log.end();
    }

    /**
     * TODO manage exceptions This method is called asynchronously when a
     * message is received from Joram, via the queue listener.
     * 
     * The recevied message is transform into a <code>MessageExchangeImpl</code>
     * by using the <code>Serializer</code>, and is sent to the
     * <code>Router</code> by calling <code>receive()</code>.
     * 
     * @param msg
     *            the jms message TODO processIncomingMessage :manage the
     *            reception of unknown message type. PB between the calculation
     *            of the next recipient and notion of service provider/consumer
     */
    protected void processIncomingMessage(Message msg) {
        log.start();
        if (msg instanceof ObjectMessage) {
            try {
                MessageExchangeImpl jbiMsg = serializer.jms2jbi(msg);

                // transfer the message to the router
                router.receive(jbiMsg);
            } catch (Exception e) {
                log.error(e);
            }
        } else {
            log.error("the received JMS message is not an objectmessage");
        }
        log.end();
    }

    // ----------------------------------------------------------------------
    // Fractal LifecycleController implementation
    // ----------------------------------------------------------------------

    /**
     * Send the message to the specified distant container, by serializing the
     * exhange with a <code>Serializer</code>. We retrieve the related
     * distant <code>Queue</code> with the namle of the container, transform
     * the <code>MessageExchangeImpl</code> into a <code>jms.Message</code>,
     * and send this message onto the distant queue.
     * 
     * @param messageExchange
     *            must be non null
     * @param containerName
     *            must be non null
     * @throws TransportException
     *             container not found, invalid message.
     */
    protected void sendDistant(MessageExchangeImpl messageExchange,
        String containerName) throws TransportException {
        log.start();
        try {
            // Find destination, create queue factory
            Queue remoteQueue = (Queue) context.lookup(JoramQueue
                .containerNameToQueueName(containerName));

            QueueConnection qc = joramQueue.getQueueConnectionFactory()
                .createQueueConnection(user, pwd);

            QueueSession qs = qc.createQueueSession(true, 0);
            QueueSender qsend = qs.createSender(remoteQueue);

            // set the TextMessage object with MessageExchange object
            Message msg = serializer.jbi2jms(messageExchange, qs);

            // send message
            qsend.send(msg);
            qs.commit();
            qc.close();
            // }
        } catch (Exception ex) {
            throw new TransportException(ex);
        }
        log.end();
    }

    /**
     * Send the message locally. No serialization of the message
     * 
     * @param messageExchange
     *            a messageExchange, can be null
     * @throws TransportException
     *             problem with the <code>Router</code>
     */
    protected void sendLocal(MessageExchangeImpl messageExchange)
        throws TransportException {
        log.start();
        try {
            router.receive(messageExchange);
        } catch (RoutingException e) {
            throw new TransportException(e);
        }
        log.end();
    }

    /**
     * Create a listener on our <code>jms.Queue</code>
     * 
     * @throws JBIException
     */
    protected void startJMS() throws JBIException {
        log.start();
        try {
            MessageListener aML = new MessageListener() {
                public void onMessage(Message aMessage) {
                    processIncomingMessage(aMessage);
                }
            };
            joramQueue.startListening(aML);
        } catch (JMSException ex) {
            throw new JBIException(ex);
        }
        log.end();
    }

    /**
     * close the QueueSession (but does not close the joram server)
     * 
     * @throws JBIException
     */
    protected void stopJMS() throws JBIException {
        log.start();
        try {
            joramQueue.unbindQueue();
            joramQueue.stopListening();
            context.close();
        } catch (Exception e) {
            throw new JBIException(e);
        }
        log.end();
    }

}