/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: RepositoryImpl.java 828 2006-07-26 12:12:15Z ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.repository;

import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.util.zip.ZipException;
import java.util.zip.ZipFile;

import org.apache.commons.io.FileUtils;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.petals.PetalsException;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.petals.util.SystemUtil;
import org.objectweb.petals.util.ZipUtil;
import org.objectweb.util.monolog.api.Logger;

import static org.objectweb.petals.util.InterfaceNamesHelper.LOGGER_NAME;
import static org.objectweb.petals.util.InterfaceNamesHelper.MONOLOG_FACTORY_NAME;

/**
 * Platform Component Repository service implementation.
 * 
 * @version $Rev: 828 $ $Date: 2006-07-26 14:12:15 +0200 (mer, 26 jui 2006) $
 * @since Petals 1.0
 * @author <a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 * @author ofabre - EBM Websourcing
 * @author ddesjardins - eBMWebsourcing
 */
public class RepositoryImpl implements RepositoryService, LifeCycleController,
    BindingController {

    /**
     * Path to component install directory
     */
    private static final String INSTALL_PATH = "install";

    /**
     * Path to repository location within Petals installation directory.
     */
    private static final String REPOSITORY_PATH = "repository";

    /**
     * The complete repository directory path.
     */
    private static String repositoryDirectory;

    /**
     * Path to component working directory
     */
    private static final String WORK_PATH = "work";

    /**
     * Component lifecycle state.
     */
    protected String fcState;

    /**
     * logger wrapper
     */
    protected LoggingUtil log;

    /**
     * Monolog Logger instance.
     */
    protected Logger logger;

    /**
     * @see org.objectweb.petals.repository.RepositoryService#addPackage(String,
     *      java.net.URI)
     */
    public URI addPackage(String uuid, URI zipArchiveLocation)
        throws PetalsException {

        // don't care about replacing existing data
        if (containsPackage(uuid)) {
            removePackage(uuid);
        }

        File installationRoot = new File(getComponentRoot(uuid), INSTALL_PATH);

        File workRoot = new File(new File(installationRoot, ".."), WORK_PATH);

        // recreate dirs necessary for adding the package into the repository
        if (!installationRoot.mkdirs()) {
            throw new PetalsException(
                "Failed to create installation root dir: "
                    + installationRoot.getAbsolutePath());
        }
        // recreate dirs necessary for adding the package into the repository
        if (!workRoot.mkdirs()) {
            throw new PetalsException(
                "Failed to create installation root dir: "
                    + workRoot.getAbsolutePath());
        }

        // explode package Zip archive file into repository
        ZipFile zipArchive = null;

        try {
            File archiveFile = new File(zipArchiveLocation);
            zipArchive = new ZipFile(archiveFile);
        } catch (ZipException ze) {
            throw new PetalsException("Unable to open Zip archive file: "
                + zipArchiveLocation.toString(), ze);
        } catch (IOException ioe) {
            throw new PetalsException("Unexpected exception.", ioe);
        }

        ZipUtil.explodeIntoDirectory(zipArchive, installationRoot);
        return installationRoot.toURI();

    }

    public void bindFc(String clientItfName, Object serverItf)
        throws NoSuchInterfaceException, IllegalBindingException,
        IllegalLifeCycleException {
        if (LOGGER_NAME.equals(clientItfName)) {
            logger = (Logger) serverItf;
        } else if (MONOLOG_FACTORY_NAME.equals(clientItfName)) {
            // nothing
        } else {
            throw new NoSuchInterfaceException(clientItfName);
        }

    }

    /**
     * @see org.objectweb.petals.repository.RepositoryService#containsPackage(java.lang.String)
     */
    public boolean containsPackage(String uuid) {
        File installationRoot = getComponentRoot(uuid);
        return installationRoot.exists() && installationRoot.isDirectory();
    }

    /**
     * @see org.objectweb.petals.repository.RepositoryService#explodeSUIntoSAInstallFolder(String,
     *      URI, String)
     */
    public URI explodeSUIntoSAInstallFolder(String suId, URI suZipLocation,
        String saId) throws PetalsException {
        File installationRoot = new File(getComponentInstallRoot(saId), suId);
        // recreate dirs necessary for adding the package into the repository
        // FIXME if the directory already exists we throw a PetalsException, it
        // is not correct except if we want to avoid the installation of a SU
        // that has been previously installed
        // TODO test if this is right to
        // test the existense of the
        // directory
        if (!installationRoot.exists() && !installationRoot.mkdirs()) {
            throw new PetalsException(
                "Failed to create installation root dir: "
                    + installationRoot.getAbsolutePath());
        }
        // explode package Zip archive file into repository
        ZipFile zipArchive = null;
        try {
            File archiveFile = new File(suZipLocation);
            zipArchive = new ZipFile(archiveFile);
        } catch (ZipException ze) {
            throw new PetalsException("Unable to open Zip archive file: "
                + suZipLocation.toString(), ze);
        } catch (IOException ioe) {
            throw new PetalsException("Unexpected exception.", ioe);
        }
        ZipUtil.explodeIntoDirectory(zipArchive, installationRoot);
        return installationRoot.toURI();
    }

    public File getComponentInstallRoot(String componentId) {
        String path = getRepositoryDirectory() + File.separator + componentId
            + File.separator + INSTALL_PATH;
        return new File(path);
    }

    public File getComponentRoot(String componentId) {
        String path = getRepositoryDirectory() + File.separator + componentId;
        return new File(path);
    }

    public File getComponentWorkRoot(String componentId) {
        String path = getRepositoryDirectory() + File.separator + componentId
            + File.separator + WORK_PATH;
        return new File(path);
    }

    public String getFcState() {
        return STARTED.equals(fcState) ? STARTED : STOPPED;
    }

    public String[] listFc() {
        return new String[0];
    }

    public Object lookupFc(String clientItfName)
        throws NoSuchInterfaceException {
        if (LOGGER_NAME.equals(clientItfName)) {
            return logger;
        } else {
            throw new NoSuchInterfaceException(clientItfName);
        }
    }

    /**
     * @see org.objectweb.petals.repository.RepositoryService#removePackage(java.lang.String)
     */
    public boolean removePackage(String componentId) {
        boolean removed = true;
        File installationRoot = getComponentRoot(componentId);
        // adding component don't care about replacing existing data
        if (installationRoot.exists()) {
            try {
                FileUtils.deleteDirectory(installationRoot);
                removed = true;
            } catch (IOException e) {
                log.error("Failed to remove directory : " + installationRoot
                    + "." + e.getMessage(), e);
                removed = false;
            }
        }
        return removed;
    }

    public void startFc() throws IllegalLifeCycleException {
        init();

        fcState = STARTED;

    }

    public void stopFc() throws IllegalLifeCycleException {
        shutdown();

        fcState = STOPPED;

    }

    public void unbindFc(String clientItfName) throws NoSuchInterfaceException,
        IllegalBindingException, IllegalLifeCycleException {
        if (LOGGER_NAME.equals(clientItfName)) {
            logger = null;
        } else if (MONOLOG_FACTORY_NAME.equals(clientItfName)) {
            // nothing
        } else {
            throw new NoSuchInterfaceException(clientItfName);
        }

    }

    protected String getRepositoryDirectory() {
        if (repositoryDirectory == null) {
            repositoryDirectory = SystemUtil.getPetalsInstallDirectory()
                .getAbsolutePath()
                + File.separator + REPOSITORY_PATH;
        }
        return repositoryDirectory;
    }

    protected void init() {
        log = new LoggingUtil(logger);
        log.start();
        log.end();
    }

    protected void shutdown() {
        log.start();

        logger = null;

        log.end();
    }

}
